@@ -309,7 +309,8 @@ typedef GstFlowReturn		(*GstPadChainListFunction)	(GstPad *pad, GstObject *paren
  *
  * This function is installed on a source pad with
  * gst_pad_set_getrange_function() and can only be called on source pads after
- * they are successfully activated with gst_pad_activate_pull().
+ * they are successfully activated with gst_pad_activate_mode() with the
+ * #GST_PAD_MODE_PULL.
  *
  * @offset and @length are always given in byte units. @offset must normally be a value
  * between 0 and the length in bytes of the data available on @pad. The
@@ -547,6 +548,11 @@ struct _GstPadProbeInfo
 #define GST_PAD_PROBE_INFO_OFFSET(d)       ((d)->offset)
 #define GST_PAD_PROBE_INFO_SIZE(d)         ((d)->size)
 
+GstEvent*      gst_pad_probe_info_get_event       (GstPadProbeInfo * info);
+GstQuery*      gst_pad_probe_info_get_query       (GstPadProbeInfo * info);
+GstBuffer*     gst_pad_probe_info_get_buffer      (GstPadProbeInfo * info);
+GstBufferList* gst_pad_probe_info_get_buffer_list (GstPadProbeInfo * info);
+
 /**
  * GstPadProbeCallback:
  * @pad: the #GstPad that is blocked
